@@ -419,6 +419,12 @@ def test_repeated_group_send_with_async_to_sync(channel_layer):
         pytest.fail(f"repeated async_to_sync wrapped group_send calls raised {exc}")
 
 
+@pytest.mark.xfail(
+    reason="""
+Fails with error in redis-py: int() argument must be a string, a bytes-like
+object or a real number, not 'NoneType'. Refs: #348
+"""
+)
 @pytest.mark.asyncio
 async def test_receive_cancel(channel_layer):
     """
@@ -551,6 +557,7 @@ async def test_message_expiry__group_send(channel_layer):
             await channel_layer.receive(channel_name)
 
 
+@pytest.mark.xfail(reason="Fails with timeout. Refs: #348")
 @pytest.mark.asyncio
 async def test_message_expiry__group_send__one_channel_expires_message(channel_layer):
     expiry = 3
